@@ -23,14 +23,14 @@ import (
 	"strings"
 	"time"
 
+	"github.com/gorilla/websocket"    // BSD-2-Clause
 	"github.com/pborman/getopt/v2"    // BSD-3-Clause
 	"golang.org/x/net/idna"           // BSD-3-Clause
-	"golang.org/x/net/websocket"      // BSD-3-Clause
 	"salsa.debian.org/mdosch/xmppsrv" // BSD-2-Clause
 )
 
 const (
-	version             = "0.5.1"
+	version             = "0.5.2"
 	nsBOSH              = "urn:xmpp:alt-connections:xbosh"
 	nsC2SdTLS           = "urn:xmpp:alt-connections:tls"
 	nsC2SQuic           = "urn:xmpp:alt-connections:quic"
@@ -70,7 +70,11 @@ var (
 	// statusOK will print [OK] in green color.
 	statusOK = "Test: [\033[32mOK\033[00m]"
 	// statusNOK will print [Not OK] in red color.
-	statusNOK = "Test: [\033[31mNot OK\033[00m]"
+	statusNOK   = "Test: [\033[31mNot OK\033[00m]"
+	colorGreen  = "\033[32m"
+	colorRed    = "\033[31m"
+	colorYellow = "\033[33m"
+	colorReset  = "\033[00m"
 )
 
 func main() {
@@ -115,6 +119,10 @@ func main() {
 	if *flagNoColor || runtime.GOOS == "windows" {
 		statusOK = "Test: [OK]"
 		statusNOK = "Test: [Not OK]"
+		colorGreen = ""
+		colorRed = ""
+		colorYellow = ""
+		colorReset = ""
 	}
 
 	// If connection test is required we'll also show IPs.
@@ -909,7 +917,8 @@ func checkBOSH(server string, target string, ipv4 bool, ipv6 bool, tlsConfig *tl
 			boshOpen.From, server)
 		return
 	}
-	fmt.Println(statusOK)
+	fmt.Print(statusOK)
+	printTLSVersion(resp.TLS.Version)
 }
 
 func checkWebSocket(server string, target string, tlsConfig *tls.Config, timeout time.Duration, targetType string) {
@@ -924,6 +933,7 @@ func checkWebSocket(server string, target string, tlsConfig *tls.Config, timeout
 	}
 	var wsOpen wsOpenXML
 	var framing, from string
+	var dialer websocket.Dialer
 	switch targetType {
 	case "c2s":
 		framing = nsXMPPFraming
@@ -943,6 +953,8 @@ func checkWebSocket(server string, target string, tlsConfig *tls.Config, timeout
 		fmt.Println(err)
 		return
 	}
+	headers := http.Header{}
+	headers.Add("Origin", origURI.String())
 	wsTLSConfig := &tls.Config{
 		MinVersion:         tlsConfig.MinVersion,
 		ServerName:         wsURI.Hostname(),
@@ -956,30 +968,24 @@ func checkWebSocket(server string, target string, tlsConfig *tls.Config, timeout
 	// https://github.com/gorilla/websocket/issues/417
 	// wsTLSConfig.NextProtos = []string{"http/1.1", "h2"}
 	wsTLSConfig.NextProtos = []string{"http/1.1"}
-	wsConfig := &websocket.Config{
-		Location:  wsURI,
-		Origin:    origURI,
-		Version:   13,
-		TlsConfig: wsTLSConfig,
-	}
-	wsConfig.Protocol = append(wsConfig.Protocol, "xmpp")
+	dialer.TLSClientConfig = wsTLSConfig
+	dialer.Subprotocols = append(dialer.Subprotocols, "xmpp")
 	ctx, cancel := context.WithTimeout(context.Background(), timeout)
 	defer cancel()
-	wsConn, err := wsConfig.DialContext(ctx)
+	wsConn, resp, err := dialer.DialContext(ctx, wsURI.String(), headers)
 	if err != nil {
 		fmt.Println(statusNOK)
 		fmt.Println(err)
 		return
 	}
-	_, err = wsConn.Write([]byte(fmt.Sprintf(
+	err = wsConn.WriteMessage(websocket.TextMessage, []byte(fmt.Sprintf(
 		`<open xmlns='%s' to='%s'%s version='1.0' />`, framing, server, from)))
 	if err != nil {
 		fmt.Println(statusNOK)
 		fmt.Println(err)
 		return
 	}
-	buf := make([]byte, 4096)
-	_, err = wsConn.Read(buf)
+	_, buf, err := wsConn.ReadMessage()
 	if err != nil {
 		fmt.Println(statusNOK)
 		fmt.Println(err)
@@ -997,13 +1003,18 @@ func checkWebSocket(server string, target string, tlsConfig *tls.Config, timeout
 		fmt.Printf("Server identifies as %s instead of %s.\n",
 			wsOpen.From, server)
 		return
-	} else if !wsConn.IsClientConn() {
-		_ = wsConn.Close()
-		fmt.Println(statusNOK)
-		fmt.Println("WS connection is no client connection.")
-		return
 	}
-	fmt.Println(statusOK)
+	fmt.Print(statusOK)
+	if resp.TLS != nil {
+		printTLSVersion(resp.TLS.Version)
+	} else {
+		state := getTLS(wsConn)
+		if state != nil {
+			printTLSVersion(state.Version)
+		} else {
+			fmt.Println()
+		}
+	}
 	err = wsConn.Close()
 	if err != nil {
 		fmt.Println(err)
@@ -1066,10 +1077,22 @@ func checkCertExpiry(c *tls.Conn) {
 	start := c.ConnectionState().PeerCertificates[0].NotBefore
 	now := time.Now()
 	if now.Before(expiry) && now.After(start) {
-		fmt.Println(statusOK)
+		fmt.Print(statusOK)
 	} else {
-		fmt.Println(statusNOK)
-		fmt.Println("Valid from", start, "to", expiry)
+		fmt.Print(statusNOK)
+		fmt.Print("Valid from", start, "to", expiry)
+	}
+	printTLSVersion(c.ConnectionState().Version)
+}
+
+func printTLSVersion(tv uint16) {
+	switch tv {
+	case tls.VersionTLS13:
+		fmt.Printf(" (%s%s%s)\n", colorGreen, tls.VersionName(tv), colorReset)
+	case tls.VersionTLS12:
+		fmt.Printf(" (%s%s%s)\n", colorYellow, tls.VersionName(tv), colorReset)
+	default:
+		fmt.Printf(" (%s%s%s)\n", colorRed, tls.VersionName(tv), colorReset)
 	}
 }
 
@@ -1105,3 +1128,18 @@ func getID() string {
 	}
 	return string(id)
 }
+
+func getTLS(c any) *tls.ConnectionState {
+	for c != nil {
+		if tc, ok := c.(*tls.Conn); ok {
+			state := tc.ConnectionState()
+			return &state
+		}
+		wc, ok := c.(interface{ NetConn() net.Conn })
+		if !ok {
+			break
+		}
+		c = wc.NetConn()
+	}
+	return nil
+}
